// FLD.PRG - Field "object" subsystem.

#define NOFLD
#include "mks50.ch"

//  Field Class
STATIC nFldClassID:= NIL
#translate FName( <f> )       => <f>\[2]
#translate FName( <f>, <n> )  => <f>\[2]:= <n>
#translate FType( <f> )       => <f>\[3]
#translate FLen( <f> )        => <f>\[4]
#translate FDec( <f> )        => <f>\[5]
#translate FDescr( <f> )      => <f>\[6]
#translate FDescr( <f>, <d> ) => <f>\[6]:= <d>
#translate FPicture( <f> )    => <f>\[7]
#translate FBlock( <f> )      => <f>\[8]
#translate FBlock( <f>, <b> ) => <f>\[8]:= <b>


//....................................
FUNCTION Fld_New( cName, cType, nLen, nDec, cDescr, cPict, bBlock )
   DEFAULT nFldClassID TO ClassIDNew( "fld" ),;
           cType TO 'C',;
           nLen TO DefaultLen( cType ),;
           nDec TO 0,;
           cDescr TO "",;
           cPict TO ""
   IF ( bBlock == NIL )
      bBlock:= &( "{|v| IIf( PCount()==0,field->"+cName+",field->"+cName+":= v ) }" )
   ENDIF
RETURN( { nFldClassID,;
          cName,;
          cType,;
          nLen,;
          nDec,;
          cDescr,;
          cPict,;
          bBlock } )

//...........................................................................
FUNCTION FldObjCheck( _Field )
RETURN ( ObjCheck( _Field, {;
   {|f,n| ValType(n) == "C" .AND. !Empty(n) },;
   {|f,t| ValType(t) == "C" .AND. Len( t ) == 1 .AND. t $ "CNDLM" },;
   {|f,l| ValType(l) == "N" .AND. l >= 0 },;
   {|f,d| ValType(d) == "N" .AND. ((FType(f)=="N" .AND. d>=0) .OR. (d==0)) },;
   {|f,d| ValType(d) == "C" },;
   {|f,p| ValType(p) == "C" },;
   {|f,b| ValType(b) == "B" } } ) )

//..........................................
FUNCTION Fld_Copy( _Field, cDescr, cName )
LOCAL _NewField:= AClone( _Field )
   DEFAULT cName TO FName( _Field )
   FName( _NewField, cName )
   FDescr( _NewField, cDescr )
RETURN( _NewField )

//....................................
FUNCTION FldName( _Field, cName )
   IF ( cName <> NIL )
      FBlock( _Field, cName )
   ENDIF
RETURN( FName( _Field ) )

//....................................
FUNCTION FldType( _Field )
RETURN( FType( _Field ) )

//....................................
FUNCTION FldLen( _Field )
RETURN( FLen( _Field ) )

//....................................
FUNCTION FldDec( _Field )
RETURN( FDec( _Field ) )

//....................................
FUNCTION FldDescr( _Field )
RETURN( FDescr( _Field ) )

//....................................
FUNCTION FldPicture( _Field )
RETURN( FPict( _Field ) )

//....................................
FUNCTION FldBlock( _Field, bBlock )
   IF ( bBlock <> NIL )
      FBlock( _Field, bBlock )
   ENDIF
RETURN( FBlock( _Field ) )

//....................................
FUNCTION IsFldClass( _Field )
RETURN( ClassID( _Field ) == nFldClassID )

//....................................
STATIC FUNCTION DefaultLen( cType, nLen )
   DEFAULT nLen TO  1
   DO CASE
   CASE ( cType == "D" )
      nLen:= 8
   CASE ( cType == "M" )
      nLen:= 10
   CASE ( cType == "L" )
      nLen:= 1
   ENDCASE
RETURN( nLen )    // cType $ "CN"

//...........................................................................
FUNCTION FldVerify( _Field, i )  // Verify diskfile matches definition
LOCAL cError:= ""
   IF !( FieldName( i ) == FldName( _Field ) ) .OR.;
      !( FieldType( i ) == FldType( _Field ) ) .OR.;
      !( FieldLen( i ) == FldLen( _Field ) ) .OR.;
      !( FieldDec( i ) == FldDec( _Field ) )
      cError:= Space(3)+;
       "Field definition:  " + PadR( FldName( _Field ), 11 ) + FldType( _Field ) +;
           Str( FldLen( _Field ), 5 ) + Str( FldDec( _Field ), 3 ) + CR_LF +;
       Space(3)+;
       "DBF field on disk: " + PadR( FieldName( i ), 11 ) + FieldType( i ) +;
           Str( FieldLen( i ), 5 ) + Str( FieldDec( i ), 3 ) + CR_LF + CR_LF
   ENDIF
RETURN( cError )

//..........................................................................
// EOF: FLD.PRG

