/* MSG.PRG - Message Routines

*/
#include "mks50.ch"

#define MsgInfo( t,l,b,r,s,m,o ) { Chr(t)+Chr(l)+Chr(b)+Chr(r)+s,m,o }
#define MsgTop(mi)        Asc( mi[1] )
#define MsgLeft(mi)       Asc( SubStr( mi[1], 2, 1 ) )
#define MsgBottom(mi)     Asc( SubStr( mi[1], 3, 1 ) )
#define MsgRight(mi)      Asc( SubStr( mi[1], 4, 1 ) )
#define MsgSavedScr(mi)   SubStr( mi[1], 5 )
#define MsgText(mi)       mi[2]

#translate MsgVisible(<mi>)     =>  <mi>\[3]
#translate MsgVisible(<mi>,<x>) =>  <mi>\[3]:= <x>

#define MSG_VISIBLE TRUE

STATIC aMsgStack:= NIL

//......................................................
PROCEDURE MsgWait( cMessage )
   IF ( aMsgStack == NIL )
      aMsgStack:= StackNew()
   ENDIF
   MsgPush( cMessage )
   MsgDisplay()
   GetKey(0)
   MsgClear()
   MsgPop()
RETURN

//..........................................................
PROCEDURE MsgPush(cMsg, nTopRow, nLeftCol )
LOCAL nBottRow, nRightCol
   DEFAULT nTopRow TO 11,;
           nLeftCol TO Int( ( MaxCol()-Len(cMsg)+1 ) / 2 )-2,;
           aMsgStack TO StackNew()
   nBottRow:= nTopRow+2
   nRightCol:= nLeftCol+Len(cMsg)+3
   StackPush( aMsgStack,;
              MsgInfo( nTopRow,nLeftCol,nBottRow,nRightCol,;
                       SaveScreen(nTopRow,nLeftCol,nBottRow,nRightCol),;
                       cMsg, !MSG_VISIBLE ) )
RETURN

//..........................................................
PROCEDURE MsgPop()
   StackPop( aMsgStack )
RETURN

//.......................................................
PROCEDURE MsgDisplay()
LOCAL aMsg:= StackTop( aMsgStack )
   IF !MsgVisible( aMsg )
      MsgDraw( aMsg )
      MsgVisible( aMsg, TRUE )
   ENDIF
RETURN

//.......................................................
PROCEDURE MsgClear()
LOCAL aMsg:= StackTop( aMsgStack )
   IF MsgVisible( aMsg )
      MsgUnDraw( aMsg )
      MsgVisible( aMsg, FALSE )
   ENDIF
RETURN

//.......................................................
PROCEDURE MsgRemove()
LOCAL aTmpStack:= StackNew(), aMsg
   WHILE !Empty( aMsgStack )
      StackPush( aTmpStack, aMsg:= StackPop( aMsgStack ) )
      IF MsgVisible( aMsg )
         MsgUnDraw( aMsg )
      ENDIF
   ENDDO
   WHILE !Empty( aTmpStack )
      StackPush( aMsgStack, StackPop( aTmpStack ) )
   ENDDO
RETURN

//.......................................................
PROCEDURE MsgRestore()
LOCAL aTmpStack:= StackNew(), aMsg
   WHILE !Empty( aMsgStack )
      StackPush( aTmpStack, aMsg:= StackPop( aMsgStack ) )
   ENDDO
   WHILE !Empty( aTmpStack )
      StackPush( aMsgStack, StackPop( aTmpStack ) )
      IF !MsgVisible( aMsg )
         MsgDraw( aMsg )
      ENDIF
   ENDDO
RETURN

//.......................................................
STATIC PROCEDURE MsgDraw( aMsg )
   PushColor( ColorMessage() )
   @ MsgTop(aMsg), MsgLeft(aMsg) CLEAR TO MsgBottom(aMsg), MsgRight(aMsg)
   @ MsgTop(aMsg), MsgLeft(aMsg) TO MsgBottom(aMsg), MsgRight(aMsg) DOUBLE
   @ MsgTop(aMsg)+1, MsgLeft(aMsg)+2 SAY MsgText( aMsg )
   PopColor()
RETURN

//.......................................................
STATIC PROCEDURE MsgUnDraw( aMsg )
   RestScreen( MsgTop( aMsg ), MsgLeft( aMsg ),;
               MsgBottom( aMsg ), MsgRight( aMsg ),;
               MsgSavedScr( aMsg ) )
RETURN


//.......................................................
// EOF: MSG.PRG
