/* PPGLOBAL.PRG - Push and Pop Global Settings in Clipper 5.0

   The functions PushGlobals() and PopGlobals() maintain an internal
   stack of Global variable settings. They therefore can be nested
   with no problem.

   SWITCHES: Must compile with "/n."

*/

STATIC aGlobalStack:= {}

//..............................................
PROCEDURE PushGlobals()
   AAdd( aGlobalStack, SaveGlobals() )
RETURN

//..............................................
PROCEDURE PopGlobals()
LOCAL nGSDepth:= Len( aGlobalStack )
   IF (nGSDepth > 0)
      RestoreGlobals( aGlobalStack[ nGSDepth ] )
      ASize( aGlobalStack, nGSDepth-1 )
   ENDIF
RETURN

//..............................................
STATIC FUNCTION OKSet(i)
RETURN ( i < 12 ).OR.( i > 15 )

//..............................................
STATIC FUNCTION SaveGlobals()
LOCAL i, aSettings[_SET_COUNT]
   FOR i= 1 TO _SET_COUNT
      IF OKSet(i)
         aSettings[i]:= Set( i )
      ENDIF
   NEXT
RETURN(aSettings)

//..............................................
STATIC PROCEDURE RestoreGlobals(aSettings)
LOCAL i
   FOR i= 1 TO _SET_COUNT
      IF OKSet(i)
         Set( i, aSettings[i] )
      ENDIF
   NEXT
RETURN

//..............................................
// EOF: PPGLOBAL.PRG
