/* EDITOR'S NOTE:
  
   WARNING!

   Use of this module is not encouraged by Nantucket.  You should be 
   aware that this is dependent upon the nature of the DBFNTX driver
   and may not (probably will not) work with other database drivers.
   It uses techniques the work under 5.0, but will not necessarily
   work under future versions of the compiler.

*/


/* WA.PRG - Create an work area editing buffer using a database file.

   This module contains two functions for export to the linker:

      WACreate()
      WACopy()

   The first of the two, WACreate() is passed an optional structure of the
   type expected by DBCreate() and returns the numeric work area of the
   memory editing buffer. It creates this area by reassigning the actual
   file handle for the database to an opened copy of NUL, the bit bucket.
   In this way, the technique occupies only one handle (for as many buffers
   as you need) on the local machine and no handles on a file server.
   Editing also does not write to the disk.

   The second, WACopy(), is a simple generic copy routine that expects a
   numeric work area to copy from. It can be used with any two work areas
   that contain the same fields.

   ***** NOTICE *****
   Currently these routines may not be fully debugged. Therefore be careful
   when using them. If you find and isolate a bug, please contact me so I
   may update this file. Thank you.

   Mike Schinkel
   800-486-5201 / 703-671-3003
   Compuserve number 73567,3612

   Copyright (c) 1990; Mike Schinkel
   All Rights Reserved

*/
#include "mks50.ch"

// Abstract and Encapsulate "WA pairs" with psuedo-functions
#define  WAArea( p )     p[1]
#define  WAFile( p )     p[2]
#define  WAErase( p )    p[3]
#define  WAUnit( a, f, e )  { a, f, e }

STATIC nNulHandle:= NIL,;        // Keep the NUL file handle.
       aUnits:= NIL              // Keep a list of area/file pairs.

//......................................................
PROCEDURE WACopy( nFromArea, nToArea )
LOCAL i, nFields, nOldArea:= Select()
   IF ( ValType( nToArea ) == "N" )
      SELECT ( nToArea )
   ENDIF
   nFields:= FCount()
   FOR i:= 1 TO nFields
      FieldPut( i, ( nFromArea )->( FieldGet( i ) ) )
   NEXT
   SELECT ( nOldArea )
RETURN

//......................................................
FUNCTION WACreate( aDBStru )
LOCAL nOldArea:= Select(),;
      cFile, nWAArea:= -1,;
      aTmp, lErase:= FALSE
   DEFAULT aDBStru TO DBStruct()
   cAlias:= UniqueAlias()
   IF !File( cFile:= Alias()+".DBF" )
      DBCreate( cFile:= UniqueFile()+".DBF", aDBStru )
      lErase:= TRUE
   ENDIF
   IF ( nNulHandle == NIL )
      nNulHandle:= FGetNext()
      aUnits:= ListNew()
   ELSE
      FClose( nNulHandle )
      aTmp:= ForceHandleNext( nNulHandle )
   ENDIF
   IF ( nNulHandle > -1 )
      USE ( cFile ) SHARED NEW ALIAS ( cAlias )
      APPEND BLANK
      nWAArea:= Select()
      FClose( nNulHandle )
      FOpen( "NUL", 2 )
      ListAdd( aUnits, WAUnit( nWAArea, cFile, lErase ) )
      IF !Empty( aTmp )
         ClearHandles( aTmp )
      ENDIF
   ENDIF
   SELECT ( nOldArea )           // Back in the USSR...
RETURN( nWAArea )

//......................................................
STATIC FUNCTION UniqueAlias()
LOCAL cAlias:="", n:= 1, cZeros:= Replicate( "0", 10 )
   WHILE Select( cAlias:= "A" + Right( cZeros + LStr( n++ ), 10 ) ) > 0
   ENDDO
RETURN( cAlias )

//......................................................
STATIC FUNCTION ForceHandleNext( nHandle )
LOCAL aHandles:= {}, nCnt
   WHILE ( AAdd( aHandles, FOpen( "NUL", 0 ) ) < nHandle )
   ENDDO
   FClose( aHandles[ nCnt:= Len( aHandles ) ] )
   ASize( aHandles, --nCnt )
RETURN( aHandles )   // Array of handles can be closed after opening file

//......................................................
STATIC PROCEDURE ClearHandles( aHandles )
LOCAL nCnt:= Len( aHandles )
   WHILE ( nCnt > 0 )
      FClose( aHandles[ nCnt-- ] )
   ENDDO
   ASize( aHandles, 0 )
RETURN

//......................................................
// EOF: WA.PRG
